@@ -1,9 +1,10 @@
 $(function() {
   var conn = new io.connect('http://' + window.location.host);
   var multiProject = false;
+  var instanceID = null;
 
   function processCarriageReturns(txt) {
-    return txt.replace(/[^\n]*\r(?!\n)/g, "");
+    return txt.replace(/[^\n]*\r(?!\n|$)/g, "");
   }
 
   conn.on('connect', function() {
@@ -15,21 +16,33 @@ $(function() {
     div.id = 'connection-error';
     div.innerHTML = 'There is no connection with the warrior.';
     document.body.insertBefore(div, document.body.firstChild);
+  });
 
-    conn.socket.reconnect();
+  conn.on('instance_id', function(msg) {
+    // we are connected to a different instance
+    if (instanceID && instanceID != msg) {
+      window.location.reload();
+    } else {
+      instanceID = msg;
+    }
   });
 
   conn.on('warrior.settings_update', function(msg) {
+    if (msg.session_id && msg.session_id != conn.socket.sessionid) return;
     reloadSettingsTab();
   });
 
   conn.on('warrior.projects_loaded', function(msg) {
+    if (msg.session_id && msg.session_id != conn.socket.sessionid) return;
+
     multiProject = true;
     $(document.body).removeClass('single-project');
     reloadProjectsTab();
   });
 
   conn.on('warrior.project_installing', function(msg) {
+    if (msg.session_id && msg.session_id != conn.socket.sessionid) return;
+
     var projectLi = $('#project-' + msg.project.name);
     projectLi.addClass('installing');
     $('div.select', projectLi).append('<span class="installing">Preparing project...</span>');
@@ -37,6 +50,8 @@ $(function() {
   });
 
   conn.on('warrior.project_installed', function(msg) {
+    if (msg.session_id && msg.session_id != conn.socket.sessionid) return;
+
     var projectLi = $('#project-' + msg.project.name);
     projectLi.removeClass('installing');
     $('div.select span.installing', projectLi).remove();
@@ -44,6 +59,8 @@ $(function() {
   });
 
   conn.on('warrior.project_installation_failed', function(msg) {
+    if (msg.session_id && msg.session_id != conn.socket.sessionid) return;
+
     var projectLi = $('#project-' + msg.project.name);
     projectLi.removeClass('installing');
     $('div.select span.installing', projectLi).remove();
@@ -53,11 +70,14 @@ $(function() {
   });
 
   conn.on('warrior.project_selected', function(msg) { // project
+    if (msg.session_id && msg.session_id != conn.socket.sessionid) return;
     reloadProjectsTab();
   });
 
   conn.on('project.refresh', function(msg) { // project, pipeline, items
     if (msg) {
+      if (msg.session_id && msg.session_id != conn.socket.sessionid) return;
+
       for (var i=0; i<msg.items.length; i++) {
         addItem(msg.items[i], true);
       }
@@ -70,6 +90,8 @@ $(function() {
   var currentWarriorStatus = null;
 
   conn.on('warrior.status', function(msg) {
+    if (msg.session_id && msg.session_id != conn.socket.sessionid) return;
+
     currentWarriorStatus = msg.status;
     showWarriorStatus(msg.status);
     if (msg.status == 'INVALID_SETTINGS') {
@@ -82,14 +104,18 @@ $(function() {
   });
 
   conn.on('runner.status', function(msg) { // project_id
+    if (msg.session_id && msg.session_id != conn.socket.sessionid) return;
     showRunnerStatus(msg.status);
   });
 
   conn.on('pipeline.start_item', function(msg) { // pipeline_id, item
+    if (msg.session_id && msg.session_id != conn.socket.sessionid) return;
     addItem(msg.item);
   });
 
   conn.on('item.output', function(msg) { // item_id, data
+    if (msg.session_id && msg.session_id != conn.socket.sessionid) return;
+
     var itemLog = $('#item-' + msg.item_id + ' pre.log')[0];
     if (itemLog) {
       if (itemLog.data) {
@@ -97,38 +123,52 @@ $(function() {
         itemLog.firstChild.nodeValue = itemLog.data;
       } else {
         itemLog.data = processCarriageReturns(msg.data);
-        itemLog.empty();
+        $(itemLog).empty();
         itemLog.appendChild(document.createTextNode(msg.data));
       }
       itemLog.scrollTop = itemLog.scrollHeight + 1000;
     }
+    updateBriefLog(msg.item_id, msg.data);
   });
 
   conn.on('item.task_status', function(msg) { // item_id, task_id, new_status, old_status
+    if (msg.session_id && msg.session_id != conn.socket.sessionid) return;
+
     var itemTask = $('#item-' + msg.item_id + ' li.task-' + msg.task_id)[0];
     if (itemTask) {
       itemTask.className = 'task-' + msg.task_id + ' ' + msg.new_status;
       $('span.s', itemTask).text(taskStatusChars[msg.new_status] || '');
+      updateBriefTasks(msg.item_id,
+                       $(itemTask).data('index'),
+                       $('#item-' + msg.item_id + ' li').length);
     }
   });
 
   conn.on('item.update_name', function(msg) { // item_id, new_name
-    $('#item-' + msg.item_id + ' h3').text(msg.new_name);
+    if (msg.session_id && msg.session_id != conn.socket.sessionid) return;
+
+    $('#item-' + msg.item_id + ' h3 .name').text(msg.new_name);
   });
 
   conn.on('item.complete', function(msg) { // pipeline_id, item_id
+    if (msg.session_id && msg.session_id != conn.socket.sessionid) return;
+
     $('#item-' + msg.item_id).addClass(itemStatusClassName['completed']);
     $('#item-' + msg.item_id + ' div.status').text(itemStatusTexts['completed']);
     scheduleDelete(msg.item_id);
   });
 
   conn.on('item.fail', function(msg) { // pipeline_id, item_id
+    if (msg.session_id && msg.session_id != conn.socket.sessionid) return;
+
     $('#item-' + msg.item_id).addClass(itemStatusClassName['failed']);
     $('#item-' + msg.item_id + ' div.status').text(itemStatusTexts['failed']);
     scheduleDelete(msg.item_id);
   });
 
   conn.on('item.cancel', function(msg) { // pipeline_id, item_id
+    if (msg.session_id && msg.session_id != conn.socket.sessionid) return;
+
     $('#item-' + msg.item_id).addClass(itemStatusClassName['canceled']);
     $('#item-' + msg.item_id + ' div.status').text(itemStatusTexts['canceled']);
     scheduleDelete(msg.item_id);
@@ -180,7 +220,8 @@ $(function() {
     'RUNNING_PROJECT': ['The warrior is working on a project.', 'Shut down', '/api/stop'],
     'SWITCHING_PROJECT': ['The warrior will switch to a different project.', 'Shut down', '/api/stop'],
     'STARTING_PROJECT': ['The warrior is beginning work on a project.', 'Shut down', '/api/stop'],
-    'SHUTTING_DOWN': ['The warrior is stopping and shutting down.', 'Keep running', '/api/keep_running'],
+    'SHUTTING_DOWN': ['The warrior is stopping and shutting down.', 'Keep running', '/api/keep_running',
+                      'Stop immediately', '/api/stop_now'],
     'REBOOTING': ['The warrior is stopping and restarting.', 'Keep running', '/api/keep_running']
   };
 
@@ -190,12 +231,20 @@ $(function() {
       $('#warrior-status-description').text(s[0]);
       $('#warrior-status-form .button-link').text(s[1]);
       $('#warrior-status-form').attr('action', s[2]);
+      if (s[3]) {
+        $('#warrior-status-form-force').css('display', 'block');
+        $('#warrior-status-form-force .button-link').text(s[3]);
+        $('#warrior-status-form-force').attr('action', s[4]);
+      } else {
+        $('#warrior-status-form-force').css('display', 'none');
+      }
     }
   }
 
   var runnerStatus = {
     'running':  ['The runner is running.', 'Stop', '/api/stop'],
-    'stopping': ['The runner is stopping.', 'Keep running', '/api/keep_running']
+    'stopping': ['The runner is stopping.', 'Keep running', '/api/keep_running',
+                 'Stop immediately', '/api/stop_now']
   };
 
   function showRunnerStatus(status) {
@@ -205,6 +254,13 @@ $(function() {
         $('#warrior-status-description').text(s[0]);
         $('#warrior-status-form .button-link').text(s[1]);
         $('#warrior-status-form').attr('action', s[2]);
+        if (s[3]) {
+          $('#warrior-status-form-force').css('display', 'block');
+          $('#warrior-status-form-force .button-link').text(s[3]);
+          $('#warrior-status-form-force').attr('action', s[4]);
+        } else {
+          $('#warrior-status-form-force').css('display', 'none');
+        }
       }
     }
   }
@@ -220,6 +276,27 @@ $(function() {
 
     $('#project-header').html(project.project_html);
 
+    if (localStorage) {
+        $('#project-header').append($("<input>", { id: "collapse-all",
+                                                   type: "checkbox",
+                                                   checked: localStorage.getItem("collapse-all") == "true" }),
+                                    $("<label>", { for: "collapse-all",
+                                                   text: "Collapse all items" }));
+        $("#collapse-all").on('change',
+                              function () {
+                                  var isCollapsed = !!$(this).attr("checked");
+                                  localStorage.setItem("collapse-all",
+                                                       isCollapsed);
+                                  if (isCollapsed)
+                                      $(".item").removeClass('open')
+                                                .addClass('closed');
+                                  else
+                                      $(".item").removeClass('closed')
+                                                .addClass('open');
+                              })
+                          .trigger('change');
+    }
+
     if (project.utc_deadline) {
       projectCountdown = new Countdown(project.utc_deadline, 'project-header');
       $('#project-header').append(projectCountdown.buildTable());
@@ -262,15 +339,23 @@ $(function() {
   }
 
   function addItem(item, skipAnimation) {
-    var itemDiv, h3, div, ol, li, span, pre,
+    var itemDiv, h3, div, ol, li, span, pre, name, briefTasks, briefLog,
         i, task;
 
+    var openOrClosed = 'open';
+    if (localStorage && localStorage.getItem("collapse-all") == "true") {
+      openOrClosed = 'closed';
+    }
+
     itemDiv = document.createElement('div');
     itemDiv.id = 'item-' + item.id;
-    itemDiv.className = 'item ' + (itemStatusClassName[item.status] || '');
+    itemDiv.className = 'item ' + openOrClosed + ' ' + (itemStatusClassName[item.status] || '');
 
     h3 = document.createElement('h3');
-    h3.appendChild(document.createTextNode(item.name));
+    $(h3).append($("<span>", { "class": 'twisty' }),
+                 $("<span>", { "class": 'name', text: item.name }),
+                 $("<span>", { "class": 'status-line' }),
+                 $("<span>", { "class": 'log-line' }));
     itemDiv.appendChild(h3);
 
     div = document.createElement('div');
@@ -285,16 +370,21 @@ $(function() {
 
     ol = document.createElement('ol');
     ol.className = 'tasks';
+    var currentTask = 0;
     for (i=0; i<item.tasks.length; i++) {
       task = item.tasks[i];
       li = document.createElement('li');
       li.className = 'task-' + task.id + ' ' + (task.status || '');
       li.appendChild(document.createTextNode(task.name + ' '));
+      $(li).data('index', i + 1);
       span = document.createElement('span');
       span.className = 's';
       span.appendChild(document.createTextNode(taskStatusChars[task.status] || ''));
       li.appendChild(span);
       ol.appendChild(li);
+      if (task.status == 'running') {
+        currentTask = i+1;
+      }
     }
     itemDiv.appendChild(ol);
 
@@ -310,7 +400,28 @@ $(function() {
     }
 
     var itemsDiv = document.getElementById('items');
-    itemsDiv.insertBefore(itemDiv, itemsDiv.firstChild)
+    itemsDiv.insertBefore(itemDiv, itemsDiv.firstChild);
+
+    updateBriefTasks(item.id, currentTask, item.tasks.length);
+    updateBriefLog(item.id, pre.data);
+  }
+
+  function updateBriefTasks(item_id, n, m) {
+    $('#item-'+ item_id +' span.status-line').text(': Step '+ n +' of '+ m +' ');
+  }
+
+  function updateBriefLog(item_id, data) {
+    var lines = data.split("\n");
+    if (lines) {
+      var line = "";
+      var len = lines.length;
+      for (var i = len - 1; i >= 0; i--) {
+        line = lines[i];
+        if (line)
+          break;
+      }
+      $('#item-'+ item_id + ' .log-line').text(line.trimLeft());
+    }
   }
 
   function scheduleAppear(item_id) {
@@ -369,6 +480,10 @@ $(function() {
     return false;
   });
 
+  $('#f-advanced-settings').click(function(e) {
+    $('#settings').toggleClass('show-advanced', e.target.checked);
+  });
+
   var Countdown = function(deadline, tableId) {
     this.deadline = deadline;
     this.tableId = tableId;
@@ -407,7 +522,6 @@ $(function() {
   };
 
   function showTab(view) {
-    console.log(currentWarriorStatus);
     if (currentWarriorStatus == 'INVALID_SETTINGS') {
       view = 'view-settings';
     }
@@ -436,6 +550,17 @@ $(function() {
 
   showTab('view-current-project');
 
+  $(".item.closed .twisty").live('click',
+                                 function () {
+                                   $(this).parent().parent().removeClass('closed')
+                                                            .addClass('open');
+                                 });
+  $(".item.open .twisty").live('click',
+                               function () {
+                                 $(this).parent().parent().removeClass('open')
+                                                          .addClass('closed');
+                               });
+
   /*
   addItem({
     'id': '1',
